unit VGA;

INTERFACE

type
  TWriteMode = (RAM,VIDRAM);

var
  WriteMode : TWriteMode;
  graphbuf : array[0..63999] of Byte;

procedure InitGraph(mode : Byte);
procedure putPixel(X,Y : Word; c : Byte);
procedure setCursor(X,Y : ShortInt);
procedure setPalette(c : Integer; r,g,b : Byte);
procedure Line(X1,Y1, X2,Y2 : Word; c : Byte);
procedure Circle(X,Y, r : Integer; c : Byte);
procedure Cls;


IMPLEMENTATION


(***************************************************)
(** SETS GRAPHMODE                                **)
(***************************************************)
procedure InitGraph(mode : Byte);

begin
  asm
    mov AH,$00
    mov AL,mode
    int $10
  end;
end;


(***************************************************)
(** SETS PAINTMODE                                **)
(***************************************************)
procedure SetMode(mode : TWriteMode);

begin
  WriteMode := mode;
end;


(***************************************************)
(** PUTS AN PIXEL (X,Y) IN COLOR C                **)
(***************************************************)
procedure putPixel(X,Y : Word; c : Byte);

begin
  if WriteMode = RAM then
    graphbuf[X+Y*320] := c
  else
    mem[$A000 : X+Y*320] := c;
end;


(***************************************************)
(** SETS CURSOR TO X,Y                            **)
(***************************************************)
procedure setCursor(X,Y : ShortInt);
begin
  asm
    mov AH,$02
    mov BH,0
    mov DH,Y
    mov DL,X
    int $10
  end;
end;


(***************************************************)
(** SETS PALETTE-INDEX C TO SPECIFIED RGB         **)
(***************************************************)
procedure setPalette(c : Integer; r,g,b : Byte);
begin
  asm
    mov BX,c
    mov DH,r
    mov CH,g
    mov CL,b
    mov AL,$10
    mov AH,$10
    int $10
  end;
end;


(***************************************************)
(** DRAWS A LINE FROM X,Y TO X1,Y1 IN COLOR C     **)
(***************************************************)
procedure Line(X1,Y1, X2,Y2 : Word; c : Byte);

var
  coX, coY, X, Y, Xmin, Ymin, Xdif, Ydif, X3, Y3 : Word;

begin
  if X1 > X2 then begin
    Xdif := X1 - X2;
    Xmin := X2;
  end else begin
    Xdif := X2 - X1;
    Xmin := X1;
  end;
  if Y1 > Y2 then begin
    Ydif := Y1 - Y2;
    Ymin := Y2;
  end else begin
    Ydif := Y2 - Y1;
    Ymin := Y1;
  end;
(* right down *)
  if ((Xmin = X1) and (Ymin = Y1)) then begin
    if Xdif > Ydif then begin
      for X := 0 to Xdif do begin
        coY := (Ydif * (X+1)) div (Xdif) + (Ymin);
        coX := X + Xmin;
        putPixel(coX,coY, c);
      end;
    end else begin
      for Y := 0 to Ydif do begin
        coX := (Xdif * (Y+1)) div (Ydif) + (Xmin);
        coY := Y + Ymin;
        putPixel(coX,coY, c);
      end;
    end;
  end;
(* left up *)
  if ((Xmin = X2) and (Ymin = Y2)) then begin
    if Xdif > Ydif then begin
      for X := 0 to Xdif do begin
        coY := ((Ydif * (X+1)) div (Xdif)) + (Ymin);
        coX := X + Xmin;
        putPixel(coX,coY, c);
      end;
    end else begin
      for Y := 0 to Ydif do begin
        coX := ((Xdif * (Y+1)) div (Ydif)) + (Xmin);
        coY := Y + Ymin;
        putPixel(coX,coY, c);
      end;
    end;
  end;
(* left down *)
  if ((Xmin = X2) and (Ymin = Y1)) then begin
    if Xdif > Ydif then begin
      for X := 0 to Xdif do begin
        coY := ((Ydif * (Xdif-X+1)) div (Xdif)) + (Ymin);
        coX := X + Xmin;
        putPixel(coX,coY, c);
      end;
    end else begin
      for Y := 0 to Ydif do begin
        coX := ((Xdif * (Y+1)) div (Ydif)) + (Xmin);
        coY := Ydif-Y + Ymin;
        putPixel(coX,coY, c);
      end;
    end;
  end;
(* right up *)
  if ((Xmin = X1) and (Ymin = Y2)) then begin
    if Xdif > Ydif then begin
      for X := 0 to Xdif do begin
        coY := ((Ydif * (Xdif-X+1)) div (Xdif)) + (Ymin);
        coX := X + Xmin;
        putPixel(coX,coY, c);
      end;
    end else begin
      for Y := 0 to Ydif do begin
        coX := ((Xdif * (Y+1)) div (Ydif)) + (Xmin);
        coY := Ydif-Y + Ymin;
        putPixel(coX,coY, c);
      end;
    end;
  end;
end;


(***************************************************)
(** DRAWS A CIRCLE AT X,Y IN COLOR C              **)
(***************************************************)
procedure Circle(X,Y, r : Integer; c : Byte);
var
  W, Wp : real;
  Xp, Yp : Integer;
begin
  w := 0;
  if r < 1 then putPixel(X,Y, c) else begin
    Wp := (pi / 6) / r;
    repeat
      Xp := round(cos(w)*r);
      Yp := round(sin(w)*r);
      putPixel(X+Xp,Y+Yp, C);
      putPixel(X+Xp,Y-Yp, C);
      putPixel(X-Xp,Y+Yp, C);
      putPixel(X-Xp,Y-Yp, C);
      w := w + Wp;
    until w > pi/2+Wp;
  end;
end;


(***************************************************)
(** CLEARS THE SCREEN (SET ALL TO COLOR 0)        **)
(***************************************************)
procedure Cls;

var
  X,Y : Integer;

begin
  for X := 0 to 320 do
    for Y := 0 to 200 do
      putPixel(X,Y, 0);
end;


END.
